/**
*
* ADTF Tutorial Filter for data processing
*
*/
#include "tutorial_filter_data_generator.h"
#include <cmath>

// The code behind the macro creates a plugin and the main entries to the plugin DLL or shared object.
// The cTutorialFilterDataGenerator will be available through the plugins class factory.
ADTF_PLUGIN("ADTF Tutorial Filter Data Generator Plugin", cTutorialFilterDataGenerator);

cTutorialFilterDataGenerator::cTutorialFilterDataGenerator()
{
    // Create our output pins with a simple plain stream type.
    m_pVelocityWriter = CreateOutputPin("velocity", stream_type_plain<float>());
    SetDescription("velocity", "Provides the generated velocity data samples");

    m_pYawRateWriter = CreateOutputPin("yawrate", stream_type_plain<float>());
    SetDescription("yawrate", "Provides the generated yawrate data samples");

    // Now create our Runner and give the ADTF Configuration Editor a hint on which
    // Active Runner to connect to it. In this case a Timer Runner.
    CreateRunner("generate_data", cTimerTriggerHint(std::chrono::seconds(1)));
    SetDescription("generate_data", "Runner to periodically trigger the function which executes the calculation");

    // And register your Property Variables to make them accessible via the
    // configuration.
    m_fVelocityChangeRate.SetDescription("Value for velocity.");
    RegisterPropertyVariable("velocity_change_rate", m_fVelocityChangeRate);
    m_fYawRateChangeRate.SetDescription("Value for yawrate.");
    RegisterPropertyVariable("yaw_rate_change_rate", m_fYawRateChangeRate);
    m_fVelocityMax.SetDescription("Maximum for velocity.");
    RegisterPropertyVariable("velocity_max_value", m_fVelocityMax);
    m_fYawRateMax.SetDescription("Maximum for yawrate.");
    RegisterPropertyVariable("yaw_rate_max_value", m_fYawRateMax);

    // set basic information about the component itself and purpose
    SetDescription("This filter shows how to generate data specified by property calibration values.");
}

// This function will be executed each time a trigger for the "generate_data" Runner occurs
tResult cTutorialFilterDataGenerator::Process(tNanoSeconds tmTrigger, IRunner* /*pRunner*/)
{
    // To create output data use the output_sample_data template
    // This will handle sample allocation for you and you can use it just
    // like its template parameter type.
    output_sample_data<float> fVelocity(tmTrigger, CalculateVelocityValue());

    // Write it to the connected sample stream.
    // Do not call any methods of output_sample_data after you have called Release() which
    // hands over the sample instance.
    m_pVelocityWriter->Write(fVelocity.Release());

    // we do the same for the yaw rate
    output_sample_data<float> fYawRate(tmTrigger, CalculateYawRateValue());
    m_pYawRateWriter->Write(fYawRate.Release());

    RETURN_NOERROR;
}

float cTutorialFilterDataGenerator::CalculateVelocityValue()
{
    // negate the velocity change rate if the max/min values are reached
    if ((m_fCurrentVelocity - std::abs(m_fVelocityChangeRate)) <= 0 ||
        m_fCurrentVelocity > m_fVelocityMax)
    {
        m_fVelocityChangeRate = -m_fVelocityChangeRate;
    }

    m_fCurrentVelocity += m_fVelocityChangeRate;

    LOG_INFO("Data Generator: Velocity: %f", m_fCurrentVelocity);

    return m_fCurrentVelocity;
}

float cTutorialFilterDataGenerator::CalculateYawRateValue()
{
    // negate the yaw rate change rate if the max/min values are reached
    if ((m_fCurrentYawRate - std::abs(m_fYawRateChangeRate)) <= 0 ||
        m_fCurrentYawRate > m_fYawRateMax)
    {
        m_fYawRateChangeRate = -m_fYawRateChangeRate;
    }

    m_fCurrentYawRate += m_fYawRateChangeRate;

    LOG_INFO("Data Generator: Yaw Rate: %f", m_fCurrentYawRate);

    return m_fCurrentYawRate;
}
